//Modified 21.05.06   for R2D3-ASSY-1432 only
// Corrections and additions  for DAB
//Refer to Programmers' Reference Guide for extended definitions
#ifndef R2D3CtlH
#define R2D3CtlH
 /*First command word structure
Bit 0-15 reserved
Bit 16   0   ,Enable interrupt on Memory full
Bit 17   0   ,Enable interrupt on Memory Empty
Bit 18   0   ,Enable interrupt on Upper threshold reached
Bit 19   0   ,Enable interrupt on Lower Threshold reached
Bit 20      Enable interrupt on Stream A Threshold reached
Bit 21      Enable interrupt on StreamB Threshold reached
Bit 22      Enable interrupt on XFIFO Full
Bit 23      Enable interrupt on XFIFO empty
Bit 24   0, Interrupt Enable Request
Bit 25   Status Request=1
Bit 26 &27 Status request 00=A,01=B,10=C,11=D
Bit 28-31 Offset address
*/
//1st Command Register constants
#define INT_EN_MEM_FULL     0x00010000  //bit 16
#define INT_EN_MEM_MT       0x00020000  //bit 17
#define INT_EN_THRESH_HI    0x00040000  //bit 18
#define INT_EN_THRESH_LO    0x00080000  //bit 19
#define INT_EN_THRESH_STRA  0x00100000  //bit 20
#define INT_EN_THRESH_STRB  0x00200000  //bit 21
#define INT_EN_XFIFO_FULL   0x00400000  //bit 22
#define INT_EN_XFIFO_MT     0x00800000  //bit 23
#define INT_EN_COMMAND      0x01000000  //bit 24

#define STATUS_A         0x00000000
#define STATUS_B         0x04000000
#define STATUS_C         0x08000000
#define STATUS_D         0x0C000000
#define OFFSET_0         0x00000000
#define OFFSET_1         0x10000000
#define OFFSET_2         0x20000000
#define OFFSET_3         0x30000000
#define OFFSET_4         0x40000000
#define OFFSET_5         0x50000000
#define STATUS_REQUEST   0x02000000

/*Second command word structure
Bit 0   Start-stop stream A, true = start
Bit 1   Start-stop stream B, true = start
Bit 2   Request stream A data transfer
Bit 3   Request stream B data transfer

Bit 4   Request XFIFO data transfer to PCI bus (Record)
Bit 5   Request XFIXO data transfer to PCI bus (Replay)
Bit 6   Stream A Buffer Reset
Bit 7   Stream B Buffer Reset

Bit 8   XFifo Reset
Bit 9   Frame Detection Mode (both streams), 0 =manual, 1=automatic
Bit 10  Stream A Record, Record=1
Bit 11  Stream B Record, Record=1

Bit 12  Stream A in E to E (Bypass)
Bit 13  Stream B in E to E (Bypass)
Bit 14  Stream A Manual Framing
Bit 15  Stream A Manual Framing
         Coded  Bit 14  Bit 15
         Unframed    0       0
         STI         1       1
         ETI         0       1
         Reserved    1       0
Bit 16  Stream B Manual Framing
Bit 17  Stream B Manual Framing
         Coded  Bit 16  Bit 17
         Unframed    0       0
         STI         1       1
         ETI         0       1
         Reserved    1       0
Bit 18  Stream A internal clock
Bit 19  Stream B internal clock

Bit 20  Stream A output inhibit (set to all zeros)
Bit 21  Stream B output inhibit (set to all zeros)
Bit 22  Card input/Output inhibited  (no HDB3 output)
Bit 23  G704 StrA

Bit 24  Stream A input impedance High=false, Low (75 ohms /120 ohms)=true
Bit 25  Stream B input impedance High=false, Low( 75 ohms /120 ohms)=true
Bit 26  Card Mode Low impedance is 120 ohms true = 120, false = 75 ohms
Bit 27  G704 StrB
Bits 28-31 Reserved
*/


//2nd Command Register constants
#define STRA_START        0x00000001
#define STRB_START        0x00000002
#define STRA_DATA_REQ     0x00000004
#define STRB_DATA_REQ     0x00000008
#define REC_XFIFO_REQ     0x00000010
#define REP_XFIFO_REQ     0x00000020
#define STRA_FIFO_RESET   0x00000040
#define STRB_FIFO_RESET   0x00000080
#define XFIFO_RESET       0x00000100
#define AUTO_FRAME        0x00000200
#define STRA_RECORD       0x00000400
#define STRB_RECORD       0x00000800
#define STRA_E2E          0x00001000
#define STRB_E2E          0x00002000

#define STRA_F0           0x00004000
#define STRA_F1           0x00008000
#define STRB_F0           0x00010000
#define STRB_F1           0x00020000
#define STRA_INTCLK       0x00040000
#define STRB_INTCLK       0x00080000
#define STRA_OP_ZERO      0x00100000
#define STRB_OP_ZERO      0x00200000
#define STR_IO_INHIBIT    0x00400000
#define STRA_G704_FRAME   0x00800000
#define STR_LOZ           0x01000000
//#define STRB_LOZ          0x02000000
#define STR_120Z          0x04000000
#define STRB_G704_FRAME   0x08000000

/*Third command word structure
Not used
Bits 0 -31 Reserved
*/


/*
4th Command Word structure
Bit 0  Stream A Lower Threshold point, LSB
Bit 1  Stream A Lower Threshold point
Bit 2  Stream A Lower Threshold point
Bit 3  Stream A Lower Threshold point
Bit 4  Stream A Lower Threshold point
Bit 5  Stream A Lower Threshold point  MSB
Bit 6  Stream A Upper Threshold point  LSB
Bit 7  Stream A Upper Threshold point
Bit 8  Stream A Upper Threshold point
Bit 9  Stream A Upper Threshold point
Bit 10 Stream A Upper Threshold point
Bit 11 Stream A Upper Threshold point MSB
Bit 12 Stream B Lower Threshold point, LSB
Bit 13 Stream B Lower Threshold point
Bit 14 Stream B Lower Threshold point
Bit 15 Stream B Lower Threshold point
Bit 16 Stream B Lower Threshold point
Bit 17 Stream B Lower Threshold point, MSB
Bit 18 Stream B Upper Threshold point, LSB
Bit 19 Stream B Upper Threshold point
Bit 20 Stream B Upper Threshold point
Bit 21 Stream B Upper Threshold point
Bit 22 Stream B Upper Threshold point
Bit 23 Stream B Upper Threshold point, MSB
Bits 24-31 Reserved
*/
#define STRA_LWR_TRIG_63  0x0000003F  //full buffer  63/64
#define STRA_LWR_TRIG_48  0x00000030  //three-quarters buffer  48/64
#define STRA_LWR_TRIG_40  0x00000028  //five-eights buffer  40/64
#define STRA_LWR_TRIG_36  0x00000024  //  36/64
#define STRA_LWR_TRIG_32  0x00000020  //half buffer  32/64
#define STRA_LWR_TRIG_31  0x0000001F  //31/64
#define STRA_LWR_TRIG_16  0x00000010  //quarter buffer  16/64
#define STRA_LWR_TRIG_8   0x00000008  //one-eighth buffer  8/64
#define STRA_LWR_TRIG_4   0x00000004  // one-sixteenth buffer  4/64
#define STRA_LWR_TRIG_0   0x00000000  //empty
#define STRA_UPR_TRIG_48  0x00000C00  //three-quarter buffer
#define STRA_UPR_TRIG_40  0x00000A00  //five-eights buffer
#define STRA_UPR_TRIG_36  0x00000900  //  36/64
#define STRA_UPR_TRIG_32  0x00000800  //half buffer
#define STRA_UPR_TRIG_31  0x000007C0  //half buffer -1
#define STRA_UPR_TRIG_16  0x00000400  //quarter buffer
#define STRA_UPR_TRIG_8   0x00000200  //one-eighth
#define STRA_UPR_TRIG_4   0x00000100  //one-sixteenth
#define STRA_UPR_TRIG_0   0x00000000   //empty
#define STRB_LWR_TRIG_48  0x00030000  //three-quarters buffer  48/64
#define STRB_LWR_TRIG_40  0x00028000  //40/64
#define STRB_LWR_TRIG_36  0x00024000  //
#define STRB_LWR_TRIG_32  0x00020000  //half buffer
#define STRB_LWR_TRIG_31  0x0001F000  //31/64
#define STRB_LWR_TRIG_16  0x00010000  //quarter buffer
#define STRB_LWR_TRIG_8   0x00008000  //one-eighth buffer
#define STRB_LWR_TRIG_4   0x00004000  // one-sixteenth buffer
#define STRB_LWR_TRIG_0   0x00000000
#define STRB_UPR_TRIG_48  0x00C00000  //three-quarter buffer
#define STRB_UPR_TRIG_40  0x00A00000  //
#define STRB_UPR_TRIG_36  0x00900000  //
#define STRB_UPR_TRIG_32  0x00800000  //half buffer
#define STRB_UPR_TRIG_31  0x007C0000  //half buffer-1
#define STRB_UPR_TRIG_16  0x00400000  //quarter buffer
#define STRB_UPR_TRIG_8   0x00200000
#define STRB_UPR_TRIG_4   0x00100000
#define STRB_UPR_TRIG_0   0x00000000
#define DEFAULT_THRESH   STRA_LWR_TRIG_32|STRA_UPR_TRIG_32|STRB_LWR_TRIG_32|STRB_UPR_TRIG_32

//Status
/* Status register 1 structure  offset 0 (1,0)
Bit 0   Cache Fifo full, true = full
Bit 1   Cache Fifo nearly empty,  true = nearly empty
Bit 2   Cache Fifo half full,  true = half full
Bit 3   0, reserved

Bit 4   Stream A  Buffer full  , Full =true
Bit 5   Stream B  Buffer full , Full = true
Bit 6   Reserved
Bit 7   Reserved

Bit 8  Stream A Buffer half-full , Half-full=true
Bit 9  Stream B Buffer half-full , Half-full =true
Bit 10  reserved
Bit 11  Reserved

Bit 12  Stream A Buffer Empty, Empty =true
Bit 13  Stream B Buffer Empty, Empty = true
Bit 14  Reserved
Bit 15  Reserved

Bit 16  Stream A Threshold triggered
Bit 17  Stream B Threshold triggered
Bit 18  Reserved
Bit 19  Reserved

Bit 20  Alarm =1
Bit 21-27 reserved (internal test)

Bit 28-31 Offset address
*/
// status register 1 constants
#define FIFO_FULL               0x00000001
#define FIFO_MT                 0x00000002
#define FIFO_HALF_FULL          0x00000004
#define STRA_BUFFER_FULL        0x00000010
#define STRB_BUFFER_FULL        0x00000020

#define STRA_BUFFER_HALF        0x00000100
#define STRB_BUFFER_HALF        0x00000200
#define STRA_BUFFER_MT          0x00001000
#define STRB_BUFFER_MT          0x00002000
#define STRA_THRESH_STATUS      0x00010000
#define STRB_THRESH_STATUS      0x00020000
#define ALARM_STATUS            0x00100000

/* Status register 2 structure
Offset 0000
Bits 0-8 Binary representation of DWORDS remaining in the XFIFO
Bits 9-27 Reserved
Bits 28-31 Offset address

*/
//Status register 2  offset 0000
#define XFIFO_WORDS_MASK    0x000001FF     //9 bits


/*
Status 2
Offset 0001
bits 0-17 Words remaining Stream A
bits 18-27 reserved
bits 28-31 Offset address
*/

#define WORDS_REMAIN_MASK   0x0003FFFF

/*Status 2
Offset 0010
bits 0-17 Words remaining Stream B
bits 18-27 reserved
Bits 28-31  Offset
*/

/*
Status 2
Offset 0011  , Thresholds
Bit 0  Stream A Lower Threshold point, LSB
Bit 1  Stream A Lower Threshold point
Bit 2  Stream A Lower Threshold point
Bit 3  Stream A Lower Threshold point
Bit 4  Stream A Lower Threshold point
Bit 5  Stream A Lower Threshold point  MSB
Bit 6  Stream A Upper Threshold point  LSB
Bit 7  Stream A Upper Threshold point
Bit 8  Stream A Upper Threshold point
Bit 9  Stream A Upper Threshold point
Bit 10 Stream A Upper Threshold point
Bit 11 Stream A Upper Threshold point MSB
Bit 12 Stream B Lower Threshold point, LSB
Bit 13 Stream B Lower Threshold point
Bit 14 Stream B Lower Threshold point
Bit 15 Stream B Lower Threshold point
Bit 16 Stream B Lower Threshold point
Bit 17 Stream B Lower Threshold point, MSB
Bit 18 Stream B Upper Threshold point, LSB
Bit 19 Stream B Upper Threshold point
Bit 20 Stream B Upper Threshold point
Bit 21 Stream B Upper Threshold point
Bit 22 Stream B Upper Threshold point
Bit 23 Stream B Upper Threshold point, MSB
Bits 24-31 Reserved
*/


/*
Status 3 Offset 0  , Signal status

Bit 0  Stream A Signal Not Present, Not present =1
Bit 1  Stream B Signal Not Present, Not present =1
Bit 2  Reserved
Bit 3  Reserved

Bit 4  Stream A Signal All Ones (AIS), AIS =1
Bit 5  Stream B Signal All Ones (AIS), AIS =1
Bit 6  Stream A Frame detected
Bit 7  Stream B Frame detected

Bit 8  Stream A Frame type
Bit 9  Stream A Frame type
         Coded   Bit 8   Bit 9
         No frame    0       0
         STI Fsync   1       1
         ETI Fsync   0       1
         Reserved    1       0
Bit 10 Stream B Frame type
Bit 11 Stream B Frame type
         Coded   Bit 10  Bit 11
         No Frame    0       0
         STI Fsync   1       1
         ETI Fsync   0       1
         Reserved    1       0

Bit 12 Stream A code violation
Bit 13 Stream B code violation
Bit 14 Reserved
Bit 15 Reserved

Bit 16 Stream A Reference clock failed , Error =1
Bit 17 Stream B Reference Clock failed , Error =1
Bit 18 Reserved
Bit 19 Reserved

Bit 20 Output driver failure monitor, true =failed
Bit 21 Reserved
Bit 22 Reserved
Bit 23 Reserved

Bit 24 Data Transfer request not understood (Bits 2 & 3 of Command B set)
Bit 25 Mode transfer request not understood (Bits 4 & 5 of Command B set)
Bit 26 Reserved
Bit 27 Hardware error, error =1

Bits 28-31 Offset address
*/
#define STRA_NO_SIGNAL    0x00000001
#define STRB_NO_SIGNAL    0x00000002
#define STRA_AIS          0x00000010
#define STRB_AIS          0x00000020
#define STRA_FAW          0x00000040
#define STRB_FAW          0x00000080
#define STRA_SIG_F0       0x00000100
#define STRA_SIG_F1       0x00000200
#define STRB_SIG_F0       0x00000400
#define STRB_SIG_F1       0x00000800
#define STRA_STREAM_ERROR 0x00001000
#define STRB_STREAM_ERROR 0x00002000
#define STRA_CLK_ERROR    0x00010000
#define STRB_CLK_ERROR    0x00020000
#define DRIVER_ERROR      0x00100000
#define DATA_REQ_ERROR    0x01000000
#define MODE_REQ_ERROR    0x02000000
#define HARDWARE_ERROR    0x08000000

/*
Status word 3
Offset 0001      Legacy
Bits 0-27 reserved (all zero)
Bits 28-31 offset address
*/

/*
Status word 3
Offset 0010      Card Version and ID
Bit 0   Hardware variant, LSB ,
Bit 1   Hardware variant
Bit 2   Hardware variant
Bit 3   Hardware variant, MSB
Bit 4  Card Number  LSB  Nibble 0
Bit 5  Card Number
Bit 6  Card Number
Bit 7  Card Number  MSB
Bit 8  Card Number  LSB  Nibble 1 =0
Bit 9  Card Number                =1
Bit 10 Card Number                =0
Bit 11 Card Number  MSB Nibble 1  =0
Bit 12 Card Number  LSB Nibble 2  =1
Bit 13 Card Number                =1
Bit 14 Card Number                =0
Bit 15 Card Number MSB  Nibble 2  =0
Bit 16  Card Number LSB Nibble 3 = 0
Bit 17  Card Number               =0
Bit 18  card Number               =1
Bit 19  Card Number MSB Nibble 3  =0
Bit 20  Card Number LSB Nibble 4  =1
Bit 21  Card Number               =0
Bit 22  Card Number               =0
Bit 23  Card Number MSB Nibble 4  =0
Bit 24- 27 reserved
Bit 28-31 Offset code
*/

/*
Status Word 3
Offset 0011  Serial Number (LSBs)
Bits 0-27 Serial Number LSBs
Bits 28-31 Offset

*/
/*
Status Word 3
Offset 0100 Serial Number (MSBs)
Bits 0-19 Serial Numbe MSBs
Bits 20-27 Reseved (all zero)
Bits 28-31 Offset
*/


/*
Status word 3
Offset 0101      Card setup Stream Status
Bit 0   Stream A active, true =active
Bit 1   Stream B active, true =active
Bit 2   Stream A Data Request,  true = requested
Bit 3   Stream B Data Request, true =requested

Bit 4   XFIFO data transfer to PCI (Record) requested, true =requested
Bit 5   XFIFO data transfer to PCI (Replay) requested, true =requested
Bit 6   Stream A Buffer reset, Reset = true
Bit 7   Stream B Buffer reset, Reset =true

Bit 8   Upper cache, (XFIFO) reset, reset =true
Bit 9   Frame Detection mode, 1= automatic (default)
Bit 10   Stream A  Record, true =record
Bit 11   Stream B Record, true =record

Bit 12  Stream A in Eto E mode, true =E to E
Bit 13  Stream B in E to E mode, true = E to E
Bit 14  Stream A Manual Framing
Bit 15  Stream A Manual Framing
         Coded  Bit 14  Bit 15
         Unframed    0       0
         ETI/NI      0       1
         STI/NI      1       1
         Reserved    1       0
Bit 16  Stream B Manual Framing
Bit 17  Stream B Manual Framing
         Coded  Bit 16  Bit 17
         Unframed    0       0
         ETI/NI      0       1
         STI/NI      1       1
         Reserved    1       0

Bit 16  Stream A unframed,        true= unframed
Bit 17  Stream B unframed,        true= unframed
Bit 18  Stream A internal clock, Internal = true
Bit 19  Stream B internal clock, Internal = true

Bit 20  Stream A output inhibited, true= inhibited
Bit 21  Stream B output inhibited, true= inhibited
Bit 22  Input/Output inhibited,  true= inhibited
Bit 23  Stream A G704

Bit 24  Stream A 75 ohm  impedance,  false= 75 ohm impedance
Bit 25  Stream B 75 ohm impedance,  false= 75 impedance
Bit 26  Impedance is  120 ohm ,  true= 120 ohm impedance
Bit 27  Stream B G 704
Bits 28-31 Offset address
*/
#define STRA_ACTIVE_STATUS       0x00000001
#define STRB_ACTIVE_STATUS       0x00000002
#define STRA_DATA_REQ_STATUS     0x00000004
#define STRB_DATA_REQ_STATUS     0x00000008
#define REC_XFIFO_REQ_STATUS     0x00000010
#define REP_XFIFO_REQ_STATUS     0x00000020
#define STRA_BUFFER_RESET_STATUS 0x00000040
#define STRB_BUFFER_RESET_STATUS 0x00000080
#define XFIFO_RESET_STATUS       0x00000100
#define AUTO_FRAME_STATUS        0x00000200
#define STRA_RECORD_STATUS       0x00000400
#define STRB_RECORD_STATUS       0x00000800
#define STRA_ETOE_STATUS         0x00001000
#define STRB_ETOE_STATUS         0x00002000
#define STRA_F0_STATUS           0x00004000
#define STRA_F1_STATUS           0x00008000
#define STRB_F0_STATUS           0x00010000    //bit 16
#define STRB_F1_STATUS           0x00020000
#define STRA_INT_CK_STATUS       0x00040000
#define STRB_INT_CK_STATUS       0x00080000
#define STRA_OP_INHIBIT_STATUS   0x00100000
#define STRB_OP_INHIBIT_STATUS   0x00200000
#define STR_IO_INHIBIT_STATUS    0x00400000
#define STRA_G704_FRAME_STATUS   0x00800000
#define STR_LOZ_STATUS           0x01000000
//reserved                       0x02000000
#define STR_120Z_STATUS          0x04000000
#define STRB_G704_FRAME_STATUS   0x08000000

/*
Status Word 4 offset 0000
/*
Bit 0  Interrupt on Buffer Full Enabled
Bit 1  Interrupt on Buffer Empty Enabled
Bit 2  Interrupt on Upper Threshold reached enabled
Bit 3  Interrupt on Lower Threshold reacehd enabled
Bit 4  Interrupt on Stream A Threshold reached enabled
Bit 5  Interrupt on Stream B Threshold reached enabled
Bit 6  Interrupt on XFIFO Full enabled
Bit 7  Interrupt on XFIFO Empty enabled
Bits 8-22 Reserved
bits 20-23 Offset address
bit 24 Interrupt caused by buffer full
bit 25 Interrupt caused by buffer empty
bit 26 Interrupt caused by Upper threshold reached
bit 27 Interrupt caused by Lower threshold reached
bit 28 Interrupt caused by Stream A
bit 29 Interrupt caused by Stream B
bit 30 Interrupt caused by XFIFO full
bit 31 Interrupt caused by XFIFO empty
*/
#define INT_EN_MEM_FULL_STATUS    0x00000001
#define INT_EN_MEM_MT_STATUS      0x00000002
#define INT_EN_THRESH_HI_STATUS   0x00000004
#define INT_EN_THRESH_LO_STATUS   0x00000008
#define INT_EN_THRESH_STR1_STATUS 0x00000010
#define INT_EN_THRESH_STR2_STATUS 0x00000020
#define INT_EN_XFIFO_FULL_STATUS  0x00000040
#define INT_EN_XFIFO_MT_STATUS    0x00000080

#define FULL_THRESH       0x01000000
#define MT_THRESH         0x02000000
#define UPPER_THRESH      0x04000000
#define LOWER_THRESH      0x08000000
#define STRA_THRESH       0x10000000
#define STRB_THRESH       0x20000000
#define XFIFO_FULL_THRESH 0x40000000
#define XFIFO_MT_THRESH   0x80000000

//Other useful defines

#define OMB1 1   //Outgoing Mailbox 1
#define OMB2 2   //Outgoing Mailbox 2
#define OMB3 3   //Outgoing Mailbox 3
#define OMB4 4   //Outgoing Mailbox 4
#define IMB1 1   //Incoming Mailbox 1
#define IMB2 2   //Incoming Mailbox 2
#define IMB3 3   //Incoming Mailbox 3
#define IMB4 4   //Incoming Mailbox 4

#define INTDATA INT_EN_THRESH_HI|INT_EN_THRESH_LO;
#define REC_MASK_BIT 0x01 //masked =true
#define REC_FRAME_BIT 0x08//framed =true
#define REC_DUAL_BIT 0x04 //dualstream=true;
#define REC_FRAME_MASK      0x00000F00
//miscellaneous masks and utility defines
#define OFFSET_MASK         0xF0000000
#define OFFSET_MASK_4       0x00F00000
#define INT_STATUS_MASK     0xFF00000
#define THRESH_A_MASK       0xFFFFF000
#define THRESH_B_MASK       0xFF000FFF
#define  SER_NUM_LWR_MASK		0x0FFFFFFF
#define  SER_NUM_UPR_MASK   0x000FFFFF
#define R2D3_BUF_SIZE 1048576 //(bytes)
//-----------------------------------------------------
#endif
