////////////////////////////////////////////////////////////////
//      AMCC5933.H - AMCC S5933 definitions
//      Copyright (c) 1996-98, BlueWater Systems,Inc.
////////////////////////////////////////////////////////////////

#ifndef _AMCC5933_H_
#define _AMCC5933_H_

    // AMCC S5933 Operations Registers
CONST ULONG AMCC5933OutgoingMailbox1            = 0x00;     // OMB1 -- Outgoing mailbox register 1
CONST ULONG AMCC5933OutgoingMailbox2            = 0x04;     // OMB2 -- Outgoing mailbox register 2
CONST ULONG AMCC5933OutgoingMailbox3            = 0x08;     // OMB3 -- Outgoing mailbox register 3
CONST ULONG AMCC5933OutgoingMailbox4            = 0x0C;     // OMB4 -- Outgoing mailbox register 4
CONST ULONG AMCC5933IncomingMailbox1            = 0x10;     // IMB1 -- Incoming mailbox register 1
CONST ULONG AMCC5933IncomingMailbox2            = 0x14;     // IMB2 -- Incoming mailbox register 2
CONST ULONG AMCC5933IncomingMailbox3            = 0x18;     // IMB3 -- Incoming mailbox register 3
CONST ULONG AMCC5933IncomingMailbox4            = 0x1C;     // IMB4 -- Incoming mailbox register 4
CONST ULONG AMCC5933FIFORegisterPort            = 0x20;     // FIFO -- FIFO register port
CONST ULONG AMCC5933MasterWriteAddress          = 0x24;     // MWAR -- Master write address register
CONST ULONG AMCC5933MasterWriteTransferCount    = 0x28;     // MWTC -- Master write transfer count
CONST ULONG AMCC5933MasterReadAddress           = 0x2C;     // MRAR -- Master read address register
CONST ULONG AMCC5933MasterReadTransferCount     = 0x30;     // MRTC -- Master read transfer count
CONST ULONG AMCC5933MailboxStatus               = 0x34;     // MBEF -- Mailbox Empty/Full Status
CONST ULONG AMCC5933InterruptControlStatus      = 0x38;     // INTCSR -- Interrupt Control/Status
CONST ULONG AMCC5933BusMasterControlStatus      = 0x3c;     // MCSR -- Bus master control/status

    // structure containing the AMCC 5933 operation registers
typedef struct _AMCC5933_OPERATION_REGISTERS {
    ULONG AMCC5933OutgoingMailbox1;                 // OMB1 -- Outgoing mailbox register 1
    ULONG AMCC5933OutgoingMailbox2;                 // OMB2 -- Outgoing mailbox register 2
    ULONG AMCC5933OutgoingMailbox3;                 // OMB3 -- Outgoing mailbox register 3
    ULONG AMCC5933OutgoingMailbox4;                 // OMB4 -- Outgoing mailbox register 4
    ULONG AMCC5933IncomingMailbox1;                 // IMB1 -- Incoming mailbox register 1
    ULONG AMCC5933IncomingMailbox2;                 // IMB2 -- Incoming mailbox register 2
    ULONG AMCC5933IncomingMailbox3;                 // IMB3 -- Incoming mailbox register 3
    ULONG AMCC5933IncomingMailbox4;                 // IMB4 -- Incoming mailbox register 4
    ULONG AMCC5933FIFORegisterPort;                 // FIFO -- FIFO register port
    ULONG AMCC5933MasterWriteAddress;               // MWAR -- Master write address register
    ULONG AMCC5933MasterWriteTransferCount;         // MWTC -- Master write transfer count
    ULONG AMCC5933MasterReadAddress;                // MRAR -- Master read address register
    ULONG AMCC5933MasterReadTransferCount;          // MRTC -- Master read transfer count
    ULONG AMCC5933MailboxStatus;                    // MBEF -- Mailbox Empty/Full Status
    ULONG AMCC5933InterruptControlStatus;           // INTCSR -- Interrupt Control/Status
    ULONG AMCC5933BusMasterControlStatus;           // MCSR -- Bus master control/status
} AMCC5933_OPERATION_REGISTERS, *PAMCC5933_OPERATION_REGISTERS;

    // RO   = Read Only
    // R/W  = Read or Write
    // R/WC = Read, Write One to Clear

    // Interrupt Control/Status Register bit definitions
#define S5933_INTCSR_32_BIT_CONVERSION   (1<<25)    // R/W
#define S5933_INTCSR_INTERRUPT_ASSERTED  (1<<23)    // RO
#define S5933_INTCSR_TARGET_ABORT        (1<<21)    // R/WC
#define S5933_INTCSR_MASTER_ABORT        (1<<20)    // R/WC
#define S5933_INTCSR_READ_COMPLETE       (1<<19)    // R/WC
#define S5933_INTCSR_WRITE_COMPLETE      (1<<18)    // R/WC
#define S5933_INTCSR_INCOMING_MESSAGE    (1<<17)    // R/WC
#define S5933_INTCSR_OUTGOING_MESSAGE    (1<<16)    // R/WC
#define S5933_INTCSR_READ_INTERRUPT      (1<<15)    // R/W
#define S5933_INTCSR_WRITE_INTERRUPT     (1<<14)    // R/W
#define S5933_INTCSR_INCOMING_INTERRUPT  (1<<12)    // R/W
#define S5933_INTCSR_IN_MAILBOX_3        (1<<11)    // R/W
#define S5933_INTCSR_IN_MAILBOX_2        (1<<10)    // R/W
#define S5933_INTCSR_IN_BYTE_2           (1<<9)     // R/W
#define S5933_INTCSR_IN_BYTE_1           (1<<8)     // R/W
#define S5933_INTCSR_OUTGOING_INTERRUPT  (1<<4)     // R/W
#define S5933_INTCSR_OUT_MAILBOX_3       (1<<3)     // R/W
#define S5933_INTCSR_OUT_MAILBOX_2       (1<<2)     // R/W

    // some useful INTCSR defines
#define S5933_INTCSR_IN_MAILBOX_4        0x00000c00 // R/W
#define S5933_INTCSR_IN_BYTE_4           0x00000300 // R/W

    // some useful INTCSR masks
#define S5933_INTCSR_INIT_MASK           0xFF000000
#define S5933_INTCSR_DMA_MASK            0xFF00FFFF
#define S5933_INTCSR_OUT_MAILBOX_MASK    0xFF00DF00
#define S5933_INTCSR_IN_MAILBOX_MASK     0xFF00C01F
#define S5933_INTCSR_INTERRUPT_MASK      0x003F0000
#define S5933_INTCSR_FIFO_CONTROL_MASK   0xFF3FDF1F
#define S5933_INTCSR_INT_SELECT_MASK     0x0000FFFF

    // Bus Master Control/Status Register bit definitions
#define S5933_MCSR_MAILBOX_FLAG_RESET    (1<<27)    // R/W
#define S5933_MCSR_ADDON_FIFO_RESET      (1<<26)    // R/W
#define S5933_MCSR_PCI_FIFO_RESET        (1<<25)    // R/W
#define S5933_MCSR_ADDON_PIN_RESET       (1<<24)    // R/W
#define S5933_MCSR_READ_TRANSFER_ENABLE  (1<<14)    // R/W
#define S5933_MCSR_READ_FIFO_SCHEME      (1<<13)    // R/W
#define S5933_MCSR_R_VS_W_PRIORITY       (1<<12)    // R/W
#define S5933_MCSR_WRITE_TRANSFER_ENABLE (1<<10)    // R/W
#define S5933_MCSR_WRITE_FIFO_SCHEME     (1<<9)     // R/W
#define S5933_MCSR_W_VS_R_PRIORITY       (1<<8)     // R/W
#define S5933_MCSR_WRITE_TRANSFER_ZERO   (1<<7)     // RO
#define S5933_MCSR_READ_TRANSFER_ZERO    (1<<6)     // RO
#define S5933_MCSR_ADDON_TO_PCI_EMPTY    (1<<5)     // RO
#define S5933_MCSR_ADDON_TO_PCI_SOME     (1<<4)     // RO
#define S5933_MCSR_ADDON_TO_PCI_FULL     (1<<3)     // RO
#define S5933_MCSR_PCI_TO_ADDON_EMPTY    (1<<2)     // RO
#define S5933_MCSR_PCI_TO_ADDON_SOME     (1<<1)     // RO
#define S5933_MCSR_PCI_TO_ADDON_FULL     (1<<0)     // RO

    // some useful MCSR definitions and masks
#define S5933_MCSR_RESET                S5933_MCSR_MAILBOX_FLAG_RESET | \
                                        S5933_MCSR_ADDON_FIFO_RESET | \
                                        S5933_MCSR_PCI_FIFO_RESET | \
                                        S5933_MCSR_ADDON_PIN_RESET

#define S5933_MCSR_FIFO_RESET           S5933_MCSR_MAILBOX_FLAG_RESET | \
                                        S5933_MCSR_ADDON_FIFO_RESET | \
                                        S5933_MCSR_PCI_FIFO_RESET

#define S5933_MCSR_DMA_MASK             0x0000FF00  // these bits need to be
                                                    // preserved when setting
                                                    // up a DMA write

    // some MBEF masks
#define S5933_MBEF_IMB4_MASK    0xF0000000
#define S5933_MBEF_IMB3_MASK    0x0F000000
#define S5933_MBEF_IMB2_MASK    0x00F00000
#define S5933_MBEF_IMB1_MASK    0x000F0000

#define S5933_MBEF_OMB4_MASK    0x0000F000
#define S5933_MBEF_OMB3_MASK    0x00000F00
#define S5933_MBEF_OMB2_MASK    0x000000F0
#define S5933_MBEF_OMB1_MASK    0x0000000F

    // NVRAM defines
#define NvramControlDataReg     0x3E    // NVRAM control/data register offset
#define NVRAMBUSY               0x8000  // NVRAM busy
#define NVRAMLOWADDRESS         0x8000  // write NVRAM low address
#define NVRAMHIGHADDRESS        0xA000  // write NVRAM High address
#define NVRAMWRITE              0xC000  // NVRAM write command 
#define NVRAMREAD               0xE000  // NVRAM read command 

#endif // _AMCC5933_H_
