/*
The WinRT driver helper functions. WinRT provides the DDK interface
for the R2D3 card to Windows applications. Data is passed through
common structures to the Windows kernel. The PCI interface consists of 
a series of Mailboxes for transfer of data. Functions in this unit 
support these transfers.
*/

/*-Includes-----------------------------------------------------------------*/
#include <stdafx.h>
#include "WinRTctl.h"
#include "R2D3Replay.h"
#include "R2D3amcc.h"     //common headers
#include "amcc5933.h"
#include "R2D3DABCtl.h"
#include <winioctl.h>

/*--------------------------------------------------------------------------*/
//////////////////////////////////////////////////////////////////////////////
//  The WinRT amcc helper functions - modified for Somerdata pci card
//AMCC.cpp  AMCC5933/AMCC5935 Sample -used by permission.
//
//
//
//////////////////////////////////////////////////////////////////////////////
/*--------------------------------------------------------------------------*/

/*-AMCC5933 Hardware definitions--------------------------------------------*/

/*-Globals------------------------------------------------------------------*/
DWORD MailboxData;

HANDLE hWinRT = NULL;							//  PCI device handle
HANDLE hWinRTInt=NULL;
HANDLE hWinRTStat=NULL;
HANDLE hWinRTBuff=NULL;
AMCC_ADDRESS    AmccAddress = {0};				//  AMCC 5933 addresses

BOOLEAN         DmaAllocated = FALSE;			//} flag to indicate DMA
												//} is allocated.

BOOLEAN         InterruptStarted = FALSE;		//} WinRt repeating interrupt
												//} started flag

WINRT_DMA_BUFFER_INFORMATION    DmaBufferInfo;	//  DMA buffer information
AMCC_REGISTERS  AmccRegisters;					//  storage for register values

int buffsize = 0;

HANDLE  AmccWinRTInterruptEvents[MAX_EVENTS] = {NULL};   // array of WinRT Events to wait on
HANDLE  AmccWin32InterruptEvents[MAX_EVENTS] = {NULL};   // For use in Win32 code

/*-Functions----------------------------------------------------------------*/

/////////////////////////////////////////////////////////////////////
//  OpenPciAmcc - Open WinRT device and any other setup for application
//  Inputs:
//  Outputs:
//  Notes:
//////////////////////////////////////////////////////////////////////
DWORD OpenPciAmcc()
{
	DWORD                       iWinRTlength;   // return length
	ULONG                       ii;             // loop counter
	DWORD                       Error;          // error return value
	WINRT_FULL_CONFIGURATION    ConfigBuf;      // configuration storage
//	char						buf[20];		// buffer for character arrays
	int							x;				// buffer for ints

	CString						buffer;			// buffer for Cstring objects
	LPTSTR						drvr = "R2D3";  // driver name

	hWinRT		= WinRTOpenNamedDevice(WINRT_T(drvr),0, true); // get  handle to device 0
	hWinRTStat	= WinRTOpenNamedDevice(WINRT_T(drvr),0, true); // get  handle to device 0
	hWinRTInt	= WinRTOpenNamedDevice(WINRT_T(drvr),0, true); // get  handle to device 0
	hWinRTBuff	= WinRTOpenNamedDevice(WINRT_T(drvr),0, true); // get  handle to device 0

	if(	hWinRT	   == INVALID_HANDLE_VALUE||
		hWinRTInt  == INVALID_HANDLE_VALUE||
		hWinRTStat == INVALID_HANDLE_VALUE||
		hWinRTBuff == INVALID_HANDLE_VALUE ) // was the WinRT device opened?
	{
		Error = GetLastError();
		buffer.AppendFormat(_T("Could not open R2D3E1 Device %d\r\n"), Error );
		theApp.dlg->display1.SetWindowText(buffer);
		return (Error);
	}

	if( WinRTGetFullConfiguration( hWinRT,                  //} get FULL
		                           &ConfigBuf,              //} configuration
			                       &iWinRTlength ) == 0 )   //} of PCI board
	{
		Error = GetLastError();

		buffer.AppendFormat(_T("WinRTGetFullConfiguration failed %d\r\n"), Error);
		theApp.dlg->display1.SetWindowText(buffer);
		return (Error);
	}

	//theApp.dlg->listBox1.ResetContent();
	buffer.AppendFormat(_T("Version %d - %d\r\n"), ConfigBuf.MajorVersion, ConfigBuf.MinorVersion);

	for (ii=0; ii<WINRT_MAXIMUM_SECTIONS; ii++) // print Port configuration
	{
		buffer.AppendFormat(_T("Port Range %X "),ii);

		x = (int)(ConfigBuf.PortStart[ii]);
		buffer.AppendFormat(_T("%X - "),x);

		x = (int)(ConfigBuf.PortStart[ii]) + (int)(ConfigBuf.PortLength[ii]);
		buffer.AppendFormat(_T("%X\r\n"),x);

		//theApp.dlg->listBox1.AddString(buffer);
	}

	for (ii=0; ii<WINRT_MAXIMUM_SECTIONS; ii++) // print Memory configuration
	{
		buffer.AppendFormat(_T("Memo Range %X "),ii);

		x = (int)(ConfigBuf.MemoryStart[ii]);
		buffer.AppendFormat(_T("%X - "),x);

		x = (int)(ConfigBuf.MemoryStart[ii]) + (int)(ConfigBuf.MemoryLength[ii]);
		buffer.AppendFormat(_T("%X\r\n"),x);

		//theApp.dlg->listBox1.AddString(buffer);
	}

        // save the AMCC5933 addresses
	AmccAddress.OperationMin = ConfigBuf.PortStart[0];
	AmccAddress.OperationMax = ConfigBuf.PortStart[0] + ConfigBuf.PortLength[0] + 1;
	AmccAddress.PassThruMin  = ConfigBuf.PortStart[1];
	AmccAddress.PassThruMax  = ConfigBuf.PortStart[1] + ConfigBuf.PortLength[1] + 1;

	    // create events
	AmccWin32InterruptEvents[ AMCCERRORINTERRUPT	] = WinRTCreateEvent(FALSE, &AmccWinRTInterruptEvents[ AMCCERRORINTERRUPT ] );
	AmccWin32InterruptEvents[ AMCCDMAWRITEINTERRUPT ] = WinRTCreateEvent(FALSE, &AmccWinRTInterruptEvents[ AMCCDMAWRITEINTERRUPT ]);
	AmccWin32InterruptEvents[ AMCCDMAREADINTERRUPT  ] = WinRTCreateEvent(FALSE, &AmccWinRTInterruptEvents[ AMCCDMAREADINTERRUPT ]);
	AmccWin32InterruptEvents[ AMCCMAILBOXREADINTERRUPT  ]= WinRTCreateEvent(FALSE, &AmccWinRTInterruptEvents[ AMCCMAILBOXREADINTERRUPT ]);
	AmccWin32InterruptEvents[ AMCCMAILBOXWRITEINTERRUPT ]= WinRTCreateEvent(FALSE, &AmccWinRTInterruptEvents[ AMCCMAILBOXWRITEINTERRUPT ]);

        // check for success
	if ((AmccWin32InterruptEvents[AMCCERRORINTERRUPT]		== NULL) ||
	    (AmccWin32InterruptEvents[AMCCDMAWRITEINTERRUPT]	== NULL) ||
		(AmccWin32InterruptEvents[AMCCDMAREADINTERRUPT]		== NULL) ||
		(AmccWin32InterruptEvents[AMCCMAILBOXREADINTERRUPT] == NULL) ||
	    (AmccWin32InterruptEvents[AMCCMAILBOXWRITEINTERRUPT]== NULL))
	{
		Error = GetLastError();
		theApp.dlg->LabelError.SetWindowText("Could not create Win32 interrupt event");
		return (Error);
	}

            // check for success
	if ((AmccWinRTInterruptEvents[AMCCERRORINTERRUPT]		== NULL) ||
	    (AmccWinRTInterruptEvents[AMCCDMAWRITEINTERRUPT]	== NULL) ||
		(AmccWinRTInterruptEvents[AMCCDMAREADINTERRUPT]		== NULL) ||
	    (AmccWinRTInterruptEvents[AMCCMAILBOXREADINTERRUPT] == NULL) ||
		(AmccWinRTInterruptEvents[AMCCMAILBOXWRITEINTERRUPT]== NULL))
	{
		Error = GetLastError();
		theApp.dlg->LabelError.SetWindowText("Could not create WinRT interrupt event ");
		return (Error);
	}


	if( WinRTSetupDmaBuffer( hWinRT,				//} setup common buffer
		                     &DmaBufferInfo,		//} this will return virtual
			                 &iWinRTlength) == 0)	//} address, length, and
													//} physical address
	{
		Error = GetLastError();
		theApp.dlg->LabelError.SetWindowText("Error allocating DMA buffer ");
		return (Error);
	}
	buffsize = DmaBufferInfo.Length;
	buffer.AppendFormat(_T("Buffer Size %d\r\n"), buffsize);
	theApp.dlg->display1.SetWindowText(buffer);
	DmaAllocated = TRUE;


	if (!SetUpAmccIsr())           // setup DMA complete repeating interrupt
	{
		theApp.dlg->LabelError.SetWindowText("SetUpAmccIsr failed");
		return (ERROR_IRQ_BUSY);
	}

	InterruptStarted = TRUE;		//} set flag indicating
									//} repeating interrupt started

	for(ii=0; ii<MAX_EVENTS; ii++)	// reset all events after enabling interrupts
	{
		ResetEvent(AmccWin32InterruptEvents[ii]);
	}

	return (ERROR_SUCCESS);
} // End OpenPciAmcc()

/*--------------------------------------------------------------------------*/

/////////////////////////////////////////////////////////////////////
//  ResetAmcc - Reset the AMCC 5933
//  Inputs:
//  Outputs:
//  Notes:
//////////////////////////////////////////////////////////////////////
DWORD ResetAmcc(void)

{
    DWORD       Error;                  // error return value
    DWORD       iWinRTlength;           // length from the driver


	{
		WINRT_CONTROL_ITEM _WinRTpp07[] =
		{//	command	param1	param2
			{OUTP_L,	0,	0},
			{OUTP_L,	0,	0},
			{OUTP_L,	0,	0},
			{OUTP_L,	0,	0},
			{OUTP_L,	0,	0},
			{OUTP_L,	0,	0},
		};
		_WinRTpp07[ 0].port  = AMCC5933BusMasterControlStatus;	//0x3C
		_WinRTpp07[ 0].value = S5933_MCSR_RESET;				//bits 24-27 set
		_WinRTpp07[ 1].port  = AMCC5933BusMasterControlStatus;
		_WinRTpp07[ 2].port  = AMCC5933MasterWriteAddress;		//0x24
		_WinRTpp07[ 3].port  = AMCC5933MasterWriteTransferCount;//0x28
		_WinRTpp07[ 4].port  = AMCC5933MasterReadAddress;		//0x2C
		_WinRTpp07[ 5].port  = AMCC5933MasterReadTransferCount; //0x30

		if (!WinRTProcessIoBufferDirect(hWinRT, _WinRTpp07,
					sizeof(_WinRTpp07), &iWinRTlength))
			goto resetAmccError;
	}


    Error = DisableAmccInterrupts(); // disable all interrupts
    return (Error);

resetAmccError:
    Error = GetLastError();
    theApp.dlg->LabelError.SetWindowText("ResetAmcc() failed " + Error);
    
    return (Error);
}// End ResetAmcc()
/*--------------------------------------------------------------------------*/

/////////////////////////////////////////////////////////////////////
//  ResetAmccFifo - Reset the AMCC 5933 Fifo
//  Inputs:
//  Outputs:
//  Notes:
//////////////////////////////////////////////////////////////////////
DWORD ResetAmccFifo(void)

{
    DWORD       Error=0;                  // error return value
    DWORD       iWinRTlength;           // length from the driver

	{
		WINRT_CONTROL_ITEM _WinRTpp07[] =
		{//	command	param1	param2
			{OUTP_L,	0,	0},
			{OUTP_L,	0,	0},
			{OUTP_L,	0,	0},
			{OUTP_L,	0,	0},
			{OUTP_L,	0,	0},
			{OUTP_L,	0,	0},
		};
		_WinRTpp07[ 0].port = AMCC5933BusMasterControlStatus; //0x3C
		_WinRTpp07[ 0].value = S5933_MCSR_PCI_FIFO_RESET|S5933_MCSR_ADDON_FIFO_RESET;//|S5933_MCSR_ADDON_PIN_RESET ; //bits 24-27 set
		_WinRTpp07[ 1].port = AMCC5933BusMasterControlStatus;
		_WinRTpp07[ 2].port = AMCC5933MasterWriteAddress; //0x24
		_WinRTpp07[ 3].port = AMCC5933MasterWriteTransferCount;//0x28
		_WinRTpp07[ 4].port = AMCC5933MasterReadAddress;  //0x2C
		_WinRTpp07[ 5].port = AMCC5933MasterReadTransferCount;//0x30
		if (!WinRTProcessIoBufferDirect(hWinRT, _WinRTpp07,
					sizeof(_WinRTpp07), &iWinRTlength))
			 goto resetAmccFifoError;

	}
	return (Error);

resetAmccFifoError:
    Error = GetLastError();
    theApp.dlg->LabelError.SetWindowText("ResetAmccFifo() failed " +Error);
    return (Error);
}// End ResetAmccFifo
/*--------------------------------------------------------------------------*/

/////////////////////////////////////////////////////////////////////
//  SetUpAmccIsr - Setup a repeating Interrupt Service  Routine (ISR)
//  Inputs:
//  Outputs:
//  Notes:This sets service routines that respond to any hardware interrupt
//////////////////////////////////////////////////////////////////////
BOOLEAN SetUpAmccIsr(void)
{
    ULONG       iWinRTlength;       // return length for WinRT functions
    ULONG       DmaCompleteMask;    // DMA interrupt Mask bits
    ULONG       AmccErrorMask;      // AMCC5933 error interrupt mask


        // AMCC5933 error interrupt mask , 0x00300000 (Bits 20 &21)
    AmccErrorMask = S5933_INTCSR_TARGET_ABORT | S5933_INTCSR_MASTER_ABORT;
        // setup DMA complete mask, 0x000C0000   (bits 18 & 19)
    DmaCompleteMask = S5933_INTCSR_READ_COMPLETE | S5933_INTCSR_WRITE_COMPLETE;

	{
	
		WINRT_CONTROL_ITEM _WinRTpp08[] =
		{//	command	param1	param2
			{DIM,	0x10040001,	   0},	// L:0 constant
			{DIM,	0x00040001,	 0x0},	// L:1 InterruptMask
			{DIM,	0x00040001,	 0x0},	// L:2 InterruptClearMask
			{DIM,	0x00040001,	 0x0},	// L:3 InterruptStatus
			{DIM,	0x00040001,	 0x0},	// L:4 Temp
			{DIM,	0x00040001,	 0x0},	// L:5 CurrentInterrupts
			{ISR_SETUP,	0x0,	0x0},
			{MATH,	0x000D0005,	0x00000000},
			{BEGIN_ISR,	0x0,	0x0},
			{WINRT_TRACE,	0x00000000,	0x0000000b},
			{NOP,	 0x0,	 0x0},
			{INP_L,	0,	0},
			{MATH,	0x000D0003,	0x00034000},
			{MATH,	0x00080004,	0x00030001},
			{MATH,	0x000D0004,	0x00041000},
			{WINRT_TRACE,	0x00000000,	0x00000021},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{LOGICAL_IF,	0x00000006,	0x00040000},
			{WINRT_TRACE,	0x00000000,	0x00000019},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{REJECT_INTERRUPT,	0x0,	0x0},
			{MATH,	0x00090005,	0x00050004},
			{MATH,	0x00080004,	0x00030002},
			{MATH,	0x000D0004,	0x00042000},
			{OUTP_L,	0,	0},
			{WINRT_TRACE,	0x00000000,	0x0000000a},
			{NOP,	 0x0,	 0x0},
			{SCHEDULE_DPC,	0x0,	0x0},
			{DIM,	0x00040001,	 0x0},	// L:32 DmaReadMask
			{DIM,	0x00040001,	 0x0},	// L:33 DmaWriteMask
			{DIM,	0x00040001,	 0x0},	// L:34 IncomingMessageMask
			{DIM,	0x00040001,	 0x0},	// L:35 OutgoingMessageMask
			{DIM,	0x00040001,	 0x0},	// L:36 AmccErrorMask
			{DIM,	0x00040001,	 0x0},	// L:37 DmaCompleteMask
			{DIM,	0x00040001,	 0x0},	// L:38 DeferredInterrupts
			{BEGIN_DPC,	0x0,	0x0},
			{WINRT_TRACE,	0x00000000,	0x0000001b},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{WINRT_BLOCK_ISR,	0x0,	0x0},
			{MATH,	0x000D0026,	0x00050000},
			{MATH,	0x000D0005,	0x00000000},
			{WINRT_RELEASE_ISR,	0x0,	0x0},
			{MATH,	0x000D0026,	0x00261000},
			{WINRT_TRACE,	0x00000000,	0x00000020},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{LOGICAL_IF,	0x00080008,	0x00260024},
			{MATH,	0x00080004,	0x00260024},
			{MATH,	0x000D0004,	0x00041000},
			{WINRT_TRACE,	0x00000000,	0x00000019},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{SET_EVENT,	0,	0},
			{LOGICAL_IF,	0x00080006,	0x00260025},
			{DMA_FLUSH,	0x0,	0x0},
			{LOGICAL_IF,	0x00080002,	0x00260021},
			{SET_EVENT,	0,	0},
			{LOGICAL_IF,	0x00080002,	0x00260020},
			{SET_EVENT,	0,	0},
			{LOGICAL_IF,	0x00080002,	0x00260022},
			{SET_EVENT,	0,	0},
			{LOGICAL_IF,	0x00080002,	0x00260023},
			{SET_EVENT,	0,	0},
			{WINRT_TRACE,	0x00000000,	0x00000018},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
		};
		_WinRTpp08[ 1].value = (ULONG)S5933_INTCSR_INTERRUPT_MASK; //0x003F0000
		_WinRTpp08[ 2].value = (ULONG)S5933_INTCSR_FIFO_CONTROL_MASK;//0xFF3FDF1F
		_WinRTpp08[ 9].value |= ((WINRT_TRACE_INFORMATIONAL)<<16);
		MEMCPY(&_WinRTpp08[10].command, "Enter ISR\n", 11);
		_WinRTpp08[11].port = AMCC5933InterruptControlStatus;
		_WinRTpp08[15].value |= ((WINRT_TRACE_INFORMATIONAL)<<16);
		MEMCPY(&_WinRTpp08[16].command, "ISR current Interrupt bits 0x%X\n", 33);
		_WinRTpp08[20].value |= ((WINRT_TRACE_INFORMATIONAL)<<16);
		MEMCPY(&_WinRTpp08[21].command, "ISR() not our interrupt\n", 25);
		_WinRTpp08[28].port = AMCC5933InterruptControlStatus;
		_WinRTpp08[29].value |= ((WINRT_TRACE_INFORMATIONAL)<<16);
		MEMCPY(&_WinRTpp08[30].command, "Exit ISR\n", 10);
		_WinRTpp08[32].value = (ULONG)S5933_INTCSR_READ_COMPLETE; //(1<<19)
		_WinRTpp08[33].value = (ULONG)S5933_INTCSR_WRITE_COMPLETE; //(1<<18)
		_WinRTpp08[34].value = (ULONG)S5933_INTCSR_INCOMING_MESSAGE;//(1<<17)
		_WinRTpp08[35].value = (ULONG)S5933_INTCSR_OUTGOING_MESSAGE;//(1<<16)
		_WinRTpp08[36].value = (ULONG)AmccErrorMask;
		_WinRTpp08[37].value = (ULONG)DmaCompleteMask;
		_WinRTpp08[40].value |=	((WINRT_TRACE_INFORMATIONAL)<<16);
		MEMCPY(&_WinRTpp08[41].command, "Enter Deferred Processing\n", 27);
		_WinRTpp08[49].value |= ((WINRT_TRACE_INFORMATIONAL)<<16);
		MEMCPY(&_WinRTpp08[50].command, "Processing Interrupt bits 0x%X\n", 32);
		_WinRTpp08[56].value |= ((WINRT_TRACE_ERROR)<<16);
		MEMCPY(&_WinRTpp08[57].command, "Got DMA error, error 0x%X\n", 25);
		_WinRTpp08[60].port = (ULONG)AmccWinRTInterruptEvents[AMCCERRORINTERRUPT];
		_WinRTpp08[64].port = (ULONG)AmccWinRTInterruptEvents[AMCCDMAWRITEINTERRUPT];
		_WinRTpp08[66].port = (ULONG)AmccWinRTInterruptEvents[AMCCDMAREADINTERRUPT];
		_WinRTpp08[68].port = (ULONG)AmccWinRTInterruptEvents[AMCCMAILBOXREADINTERRUPT];
		_WinRTpp08[70].port = (ULONG)AmccWinRTInterruptEvents[AMCCMAILBOXWRITEINTERRUPT];
		_WinRTpp08[71].value |= ((WINRT_TRACE_INFORMATIONAL)<<16);
		MEMCPY(&_WinRTpp08[72].command, "Exit Deferred Processing\n", 24);
		if (!WinRTSetInterrupt(hWinRT, _WinRTpp08,
					sizeof(_WinRTpp08), &iWinRTlength))
			 goto isrError;

	}
    return (TRUE);

isrError:
        // get interrupt error
	theApp.dlg->LabelError.SetWindowText("Error with ISR, error code " + (int)GetLastError());
    return (FALSE);

}// End SetUpAmccIsr();
/*--------------------------------------------------------------------------*/

/////////////////////////////////////////////////////////////////////
//  DisableAmccInterrupts - disable all AMCC5933 interrupts
//  Inputs:
//  Outputs:
//  Notes:
//////////////////////////////////////////////////////////////////////
DWORD DisableAmccInterrupts(void)

{
    DWORD       Error;                  // error return value
    DWORD       iWinRTlength;           // length from the driver

	{
		WINRT_CONTROL_ITEM _WinRTpp09[] =
		{//	command	param1	param2
			{DIM,	0x00040001,	 0x0},	// L:0 InterruptInitMask
			{DIM,	0x00040001,	 0x0},	// L:1 InterruptStatus
			{DIM,	0x00040001,	 0x0},	// L:2 Temp
			{INP_L,	0,	0},
			{MATH,	0x000D0001,	0x00014000},
			{MATH,	0x00080002,	0x00010000},
			{MATH,	0x000D0002,	0x00022000},
			{OUTP_L,	0,	0},
		};
		_WinRTpp09[ 0].value = (ULONG)S5933_INTCSR_INIT_MASK;
		_WinRTpp09[ 3].port = AMCC5933InterruptControlStatus;
		_WinRTpp09[ 7].port = AMCC5933InterruptControlStatus;
		if (!WinRTProcessIoBufferDirect(hWinRT, _WinRTpp09,
					sizeof(_WinRTpp09), &iWinRTlength))
			 goto disableAmccInterruptsError;

	}

    return (ERROR_SUCCESS);

disableAmccInterruptsError:
    Error = GetLastError();
    printf("DisableAmccInterrupts() failed 0x%X\n", Error);
    return (Error);
}// End DisableAmccInterrupts()
/*--------------------------------------------------------------------------*/

/////////////////////////////////////////////////////////////////////
//  EnableAmccInterrupts - enable AMCC interrupts
//  Inputs: InterruptEnable - interrupt bits to enable
//  Outputs:
//  Notes:
//////////////////////////////////////////////////////////////////////
DWORD EnableAmccInterrupts(DWORD InterruptEnable)

{
    DWORD       Error;                  // error return value
    DWORD       iWinRTlength;           // length from the driver

	{
		WINRT_CONTROL_ITEM _WinRTpp10[] =
		{//	command	param1	param2
			{DIM,	0x00040001,	 0x0},	// L:0 InterruptInitMask
			{DIM,	0x00040001,	 0x0},	// L:1 InterruptEnable
			{DIM,	0x00040001,	 0x0},	// L:2 InterruptStatus
			{DIM,	0x00040001,	 0x0},	// L:3 Temp
			{INP_L,	0,	0},
			{MATH,	0x000D0002,	0x00024000},
			{MATH,	0x00080003,	0x00020000},
			{MATH,	0x00090002,	0x00030001},
			{MATH,	0x000D0002,	0x00022000},
			{OUTP_L,	0,	0},
			{MATH,	0x000D0002,	0x00021000},
			{WINRT_TRACE,	0x00000000,	0x00000049},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
		};
		_WinRTpp10[ 0].value = (ULONG)S5933_INTCSR_INIT_MASK; //0xff000000
		_WinRTpp10[ 1].value = (ULONG)InterruptEnable;  //bits 10 & 12 set in read, bit 4 in write
		_WinRTpp10[ 4].port = AMCC5933InterruptControlStatus;
		_WinRTpp10[ 9].port = AMCC5933InterruptControlStatus;
		_WinRTpp10[11].value |= ((WINRT_TRACE_INFORMATIONAL)<<16);
		MEMCPY(&_WinRTpp10[12].command, "EnableAmccInterrupts(): Wrote ICSR with 0x%X\n", 73);
		if (!WinRTProcessIoBufferDirect(hWinRT, _WinRTpp10,
					sizeof(_WinRTpp10), &iWinRTlength))
			 goto enableAmccInterruptsError;

	}

    return (ERROR_SUCCESS);

enableAmccInterruptsError:
    Error = GetLastError();
	theApp.dlg->LabelError.SetWindowText("enableAmccInterruptsError() failed ");
    return (Error);
}// End EnableAmccInterrupts()
/*--------------------------------------------------------------------------*/

/////////////////////////////////////////////////////////////////////
//  StopInterrupt - stop WinRt repeating interrupt
//  Inputs:
//  Outputs:
//  Notes:
//////////////////////////////////////////////////////////////////////
VOID StopInterrupt(void)
{
    DWORD       iWinRTlength;       // return length

	{
		WINRT_CONTROL_ITEM _WinRTpp11[] =
		{//	command	param1	param2
			{STOP_INTERRUPT,	0x0,	0x0},
		};
		if (!WinRTProcessIoBuffer(hWinRT, _WinRTpp11,
					sizeof(_WinRTpp11), &iWinRTlength))
			 goto isrExitError;

	}

    return;

isrExitError:
    printf("StopInterrupts() failed, error code %d.\n", GetLastError());
    return;
}// End StopInterrupt()
/*--------------------------------------------------------------------------*/

void RejectInterrupt(void)
{
	DWORD       iWinRTlength;       // return length

	{
		WINRT_CONTROL_ITEM _WinRTpp11[] =
		{//	command	param1	param2
			{REJECT_INTERRUPT,	0x0,	0x0},
		};
		if (!WinRTProcessIoBuffer(hWinRT, _WinRTpp11,
					sizeof(_WinRTpp11), &iWinRTlength))
			goto isrExitError;

	}

    return;

isrExitError:
    printf("StopInterrupts() failed, error code %d.\n", GetLastError());
    return;
}// End RejectInterrupt();
/*--------------------------------------------------------------------------*/

/////////////////////////////////////////////////////////////////////
//  ReadMailBox1 - reads mail box without interrupt
//  Inputs: Address - mail box address to read
//  Outputs:
//  Notes:
//////////////////////////////////////////////////////////////////////
DWORD ReadMailBox1(DWORD Address,DWORD *pMailboxData)//,TObject *Sender)
{
	DWORD       Data;                   // mail box data
	DWORD       Error;                  // error return value
	DWORD       iWinRTlength;           // length from the driver
	CString buffer;


            // check for valid mail box
	if((Address < AMCC_MINIMUM_MAILBOX) || (Address > AMCC_MAXIMUM_MAILBOX))
	{
            // the mailbox does not exist
		buffer.Format(_T("ReadMailBox(): Invalid MailBox %d valid range %d - %d"),
											Address,
											AMCC_MINIMUM_MAILBOX,
											AMCC_MAXIMUM_MAILBOX);
		theApp.dlg->LabelError.SetWindowText(buffer);
		return (ERROR_INVALID_DATA);
	}
            // enable the correct incoming mailbox interrupt

	{
		WINRT_CONTROL_ITEM _WinRTpp03[] =
		{//	command	param1	param2
			{DIM,	0x00040001,	 0x0},	// L:0 Data
			{INP_LA,	0,	0},
			{MATH,	0x000D0000,	0x00004000},
			{WINRT_TRACE,	0x00000000,	0x0000001c},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{MATH,	0x000D0000,	0x00001000},
			{WINRT_TRACE,	0x00000000,	0x00000010},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
		};
		_WinRTpp03[ 1].port = IN_MAILBOX_ADDRESS; //macro includes address
		_WinRTpp03[ 3].port = IN_MAILBOX_ADDRESS;
		if (!WinRTProcessIoBufferDirect(hWinRT, _WinRTpp03,
				sizeof(_WinRTpp03), &iWinRTlength))
			goto readMailBoxError;
	Data = (ULONG)_WinRTpp03[ 0].value;

	}

	*pMailboxData=Data;
	return (ERROR_SUCCESS);

readMailBoxError:
    Error = GetLastError();
	theApp.dlg->LabelError.SetWindowText("ReadMailBox1() failed " + Error);
    return (Error);
}
/*--------------------------------------------------------------------------*/

/////////////////////////////////////////////////////////////////////
//  WriteMailBox1 - writes mail box   without interrupt
//  Inputs: Address - mail box address to write
//          Data - data value to write to mail box
//  Outputs:
//  Notes:
//////////////////////////////////////////////////////////////////////
DWORD WriteMailBox1(DWORD Address, DWORD Data)
{
	DWORD       Error;                  // error return value
	DWORD       iWinRTlength;           // length from the driver
	CString		buffer;					// string buffer

            // check for valid mail box
	if((Address < AMCC_MINIMUM_MAILBOX) || (Address > AMCC_MAXIMUM_MAILBOX))
	{
            // the mailbox does not exist
		buffer.Format(_T("WriteMailBox(): Invalid MailBox %d valid range %d - %d"), 
					  Address, AMCC_MINIMUM_MAILBOX, AMCC_MAXIMUM_MAILBOX);  
		theApp.dlg->LabelError.SetWindowText(buffer);
		return (ERROR_INVALID_DATA);
	}

	{
		WINRT_CONTROL_ITEM _WinRTpp04[] =
		{//	command	param1	param2
			{OUTP_LA,	0,	0},
			{WINRT_TRACE,	0x00000000,	0x0000002d},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{WINRT_TRACE,	0x00000000,	0x0000002e},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
		};
		_WinRTpp04[ 0].port = OUT_MAILBOX_ADDRESS;
		_WinRTpp04[ 0].value = Data;
		_WinRTpp04[ 1].port = OUT_MAILBOX_ADDRESS;
		if (!WinRTProcessIoBufferDirect(hWinRT, _WinRTpp04,
				sizeof(_WinRTpp04), &iWinRTlength))
			goto writeMailBoxError;

	}

	return (ERROR_SUCCESS);


writeMailBoxError:
    Error = GetLastError();
	theApp.dlg->LabelError.SetWindowText("WriteMailBox()failed "+ Error);
    return (Error);
}
/*--------------------------------------------------------------------------*/

///////////////////////////////////////////////////////////////////////////////
//  ClosePciAmcc - cleanup before exiting, close all handles. free memory, etc
//  Inputs:
//  Outputs:
//  Notes:
///////////////////////////////////////////////////////////////////////////////
DWORD ClosePciAmcc()
{
	DWORD iWinRTlength;   // return length


	(void)DisableAmccInterrupts();  // disable all AMCC interrupts

	if (InterruptStarted == TRUE)   // stop repeating interrup if started
	{
		StopInterrupt();
	}

	if (DmaAllocated == TRUE)       // free the DMA if allocated
	{
		if( WinRTFreeDmaBuffer( hWinRT,             //} free the WinRt
			                  &DmaBufferInfo,		//} common buffer
				              &iWinRTlength) == 0)	//}
		{
			theApp.dlg->LabelError.SetWindowText("WinRTDMAFreeBuffer() failed "+ GetLastError());
		}
	}

                                // close down events
	if (AmccWin32InterruptEvents[AMCCERRORINTERRUPT] != NULL )
		WinRTCloseEvent(AmccWin32InterruptEvents[AMCCERRORINTERRUPT]);
	if (AmccWin32InterruptEvents[AMCCDMAWRITEINTERRUPT] != NULL )
		WinRTCloseEvent(AmccWin32InterruptEvents[AMCCDMAWRITEINTERRUPT]);
	if (AmccWin32InterruptEvents[AMCCDMAREADINTERRUPT] != NULL )
		WinRTCloseEvent(AmccWin32InterruptEvents[AMCCDMAREADINTERRUPT]);
	if (AmccWin32InterruptEvents[AMCCMAILBOXREADINTERRUPT] != NULL )
		WinRTCloseEvent(AmccWin32InterruptEvents[AMCCMAILBOXREADINTERRUPT]);
	if (AmccWin32InterruptEvents[AMCCMAILBOXWRITEINTERRUPT] != NULL )
		WinRTCloseEvent(AmccWin32InterruptEvents[AMCCMAILBOXWRITEINTERRUPT]);

	WinRTCloseDevice(hWinRT);       // Close the WinRT Handle
	hWinRT = NULL;                  //
	WinRTCloseDevice(hWinRTStat);	// Close the WinRT Handle
	hWinRTStat = NULL;
	WinRTCloseDevice(hWinRTBuff);   // Close the WinRT Handle
	hWinRTBuff = NULL;
	WinRTCloseDevice(hWinRTInt);    // Close the WinRT Handle
	hWinRTInt = NULL;

return (ERROR_SUCCESS);
}
/*--------------------------------------------------------------------------*/
