
/*--Includes--------------------------------------------------------------*/
#pragma once
#include <stdafx.h>
#include <memory.h>
#include <winioctl.h>
#include "BufferThread.h"
#include "StatusThread.h"
#include "ControlDlg.h"
#include "InterruptThread.h"
#include "winrtctl.h"
#include "R2D3DABCtl.h"
#include "amcc5933.h"
#include "R2D3Interrupt.h"

/*--Defines---------------------------------------------------------------*/

/*--Globals---------------------------------------------------------------*/

extern HANDLE hWinRTBuff;
extern WINRT_DMA_BUFFER_INFORMATION  DmaBufferInfo; //DMA buffer information
extern HANDLE hDMAWriteCompleteEvent;
extern HANDLE hDMAReadCompleteEvent;
extern HANDLE hBufferEventA;
extern HANDLE hBufferGoEvent;
extern HANDLE hBufferEventB;
extern HANDLE hHiThreshIntEvent;
extern CBufferThread *pBufferThread;
extern CInterruptThread *pInterruptThread;



/*--Methods---------------------------------------------------------------*/

// Stub
UINT CBufferThread::mtstub1(LPVOID xthis)
{
	CBufferThread *_this=(CBufferThread *)xthis; 
	_this->Execute();
	return 0;
}
/*------------------------------------------------------------------------*/
//Constructor
CBufferThread::CBufferThread()
{
	memset(DataBuffer1, 0xAA, sizeof(DataBuffer1));
	memset(DataBuffer2, 0xBB, sizeof(DataBuffer2));
	NumBytes=65536;
	Terminated = false;
	AfxBeginThread(mtstub1, this);
}
/*------------------------------------------------------------------------*/

void CBufferThread::Execute()
{
	DWORD WaitResult=0;
	CString buffer;

	BufferPages=(R2D3_BUF_SIZE/2)/NumBytes;//set here cos wait numbytes may have changed during wait
	while(!Terminated)
	{      //wait for go button and threshold to be set
		WaitResult = WaitForSingleObject(hBufferGoEvent,INFINITE);
		WaitResult = WaitForSingleObject(hHiThreshIntEvent,6000);
		if(Terminated)
      break;
    ResetEvent(hHiThreshIntEvent);
		if(WaitResult==WAIT_OBJECT_0)//hi threshold
		{
			if(pInterruptThread->DataSourceA)
				BufferTransferA();
			if(pInterruptThread->DataSourceB)
				BufferTransferB();

			//enable incoming mailbox interrupt
			theApp.dlg->gInterruptCommand|= S5933_INTCSR_INCOMING_INTERRUPT;
			EnableAmccInterrupts(theApp.dlg->gInterruptCommand);
		}
		if(WaitResult==WAIT_TIMEOUT)//
		{
			buffer.Format(_T("Buffer Transfer: Timeout %d"), WaitResult);
			theApp.dlg->LabelError.SetWindowText(buffer);
			//theApp.dlg->LabelError.SetWindowText("Buffer Transfer: Timeout "
			//			+ WaitResult);
		}
	}
}
/*------------------------------------------------------------------------*/

void CBufferThread::BufferTransferA()
{
	short FifoBytes=256;
	int n=0;
	BufferPages=(R2D3_BUF_SIZE/2)/ NumBytes;//set here cos wait numbytes may have changed during wait

	/*CMenu menu;
	menu.LoadMenu(IDR_MENU1);
	UINT state;
	state = menu.GetMenuState(ID_WRITE, MF_BYCOMMAND);*/
	//half the card buffer size

	
	theApp.dlg->gCommand2|=STRA_DATA_REQ;
	if(theApp.dlg->Write)
		theApp.dlg->gCommand2|=REC_XFIFO_REQ;
	else
		theApp.dlg->gCommand2|=REP_XFIFO_REQ;
	WriteMailBox1(OMB2, theApp.dlg->gCommand2);

	if(theApp.dlg->Write)
	{
		ResetEvent(hDMAWriteCompleteEvent);
		for(n=0;n<BufferPages;n++)//get data from card
			ReadFromCard(NumBytes,true);
	}
	//state = menu.GetMenuState(ID_READ, MF_BYCOMMAND);
	if(theApp.dlg->Read)
	{
		for(n=0;n<BufferPages;n++)
		{ //copy data to common buffer from dummy data buffer
			WriteToCard(NumBytes,true);
		}//end of for
	}
	//state = menu.GetMenuState(ID_WRITE, MF_BYCOMMAND);
	theApp.dlg->gCommand2&=~STRA_DATA_REQ;
	if(theApp.dlg->Write)
		theApp.dlg->gCommand2&=~REC_XFIFO_REQ;
	else
		theApp.dlg->gCommand2&=~REP_XFIFO_REQ;
	WriteMailBox1(OMB2, theApp.dlg->gCommand2);
}
/*------------------------------------------------------------------------*/
void CBufferThread::BufferTransferB()
{
	short FifoBytes=256;
	int n=0;
	BufferPages=(R2D3_BUF_SIZE/2)/ NumBytes;//set here cos wait numbytes may have changed during wait

	/*CMenu menu;
	menu.LoadMenu(IDR_MENU1);
	UINT state;
	state = menu.GetMenuState(ID_WRITE, MF_BYCOMMAND);*/
	//half the card buffer size

	theApp.dlg->gCommand2|=STRB_DATA_REQ;
	if(theApp.dlg->Write)
		theApp.dlg->gCommand2|=REC_XFIFO_REQ;
	else
		theApp.dlg->gCommand2|=REP_XFIFO_REQ;
	WriteMailBox1(OMB2, theApp.dlg->gCommand2);

	if(theApp.dlg->Write)
	{
		ResetEvent(hDMAWriteCompleteEvent);
		for(n=0;n<BufferPages;n++)//get data from card
			ReadFromCard(NumBytes,true);
	}
	//state = menu.GetMenuState(ID_READ, MF_BYCOMMAND);
	if(theApp.dlg->Read)
	{
		for(n=0;n<BufferPages;n++)
		{ //copy data to common buffer from dummy data buffer
			WriteToCard(NumBytes,true);
		}
	}
	//state = menu.GetMenuState(ID_WRITE, MF_BYCOMMAND);
	theApp.dlg->gCommand2&=~STRB_DATA_REQ;
	if(theApp.dlg->Write)
		theApp.dlg->gCommand2&=~REC_XFIFO_REQ;
	else
		theApp.dlg->gCommand2&=~REP_XFIFO_REQ;
	WriteMailBox1(OMB2, theApp.dlg->gCommand2);
}

/*------------------------------------------------------------------------*/
void CBufferThread::ReadFromCard(int ReadSize,bool StreamA)
{
	DWORD WaitTimeout;
	CString buffer;
//	int n;
	DmaReadFifo(ReadSize);
	WaitTimeout=WaitForSingleObject(hDMAWriteCompleteEvent,15000);
	if(WaitTimeout==WAIT_OBJECT_0)
	{
		if(StreamA)
			CopyMemory(DataBuffer1, DmaBufferInfo.pVirtualAddress,ReadSize);
		else
			CopyMemory(DataBuffer2, DmaBufferInfo.pVirtualAddress,ReadSize);
		ResetEvent(hDMAWriteCompleteEvent);
	}
	if(WaitTimeout == WAIT_TIMEOUT )
	{      //debug messages
		buffer.Format(_T("Buffer: DMA write complete failed, %d"), WaitTimeout);
		theApp.dlg->LabelError.SetWindowText(buffer);
	}
}

/*------------------------------------------------------------------------*/
//////////////////////////////////////////////////////////////////////
//  DmaReadFifo - sets up read DMA, uses interrupt in Interrupt thread
//  Inputs: Length - Dma read length
//  Outputs:
//  Notes:
//////////////////////////////////////////////////////////////////////
DWORD CBufferThread::DmaReadFifo(DWORD Length)
{
	DWORD       Error=0;       // error return value
	DWORD       Address=0;  // address of WinRT common buffer (logical address)
	DWORD       iWinRTlength=0;     // length from the driver
	CString		buffer;


	if((Length < 4) || (Length > DmaBufferInfo.Length)) // check for valid length
	{
		buffer.Format(_T("BufferReadFifo: Invalid Length %d valid range 4-%d"), 
						  Length,
						  DmaBufferInfo.Length);
		theApp.dlg->LabelError.SetWindowText(buffer);
		return(ERROR_INVALID_DATA);
	}

	if (Length & 0x3)                   //} check that length can be
	{									//} converted into ULONG count (4byte chunks)
		theApp.dlg->LabelError.SetWindowText("DmaReadFifo: Invalid Length,  must be on a DWORD boundary");
		return (ERROR_INVALID_DATA);
	}
					// get common buffer address for DMA (logical)
	Address = (DWORD)(DmaBufferInfo.LogicalAddress);

	{
		static WINRT_CONTROL_ITEM _WinRTpp01[] =
		{//	command	param1	param2
			{DIM,	0x00040001,	 0x0},	// L:0 DmaMask
			{DIM,	0x00040001,	 0x0},	// L:1 StartDMA
			{DIM,	0x00040001,	 0x0},	// L:2 DmaControl
			{DIM,	0x00040001,	 0x0},	// L:3 Temp
			{OUTP_L,	0,	0},         //4
			{OUTP_L,	0,	0},
			{WINRT_TRACE,	0x00000000,	0x00000024},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{WINRT_TRACE,	0x00000000,	0x00000025},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{INP_L,	0,	0},
			{MATH,	0x000D0002,	0x00024000},
			{MATH,	0x00080003,	0x00020000},
			{MATH,	0x00090002,	0x00030001},
			{MATH,	0x000D0002,	0x00021000},
			{WINRT_TRACE,	0x00000000,	0x00000033},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{MATH,	0x000D0002,	0x00022000},
			{OUTP_L,	0,	0},
		};
		_WinRTpp01[ 0].value = (ULONG)S5933_MCSR_DMA_MASK;//0x0000FF00
		_WinRTpp01[ 1].value = (ULONG)S5933_MCSR_WRITE_TRANSFER_ENABLE;//(1<<10)
		_WinRTpp01[ 4].port = AMCC5933MasterWriteAddress;
		_WinRTpp01[ 4].value = Address;// address of WinRT common buffer (logical address)
		_WinRTpp01[ 5].port = AMCC5933MasterWriteTransferCount;//0x28
		_WinRTpp01[ 5].value = Length;//no. of bytes to be read
		_WinRTpp01[ 6].port = Length;//no. of bytes to be read
		_WinRTpp01[ 6].value |= ((WINRT_TRACE_INFORMATIONAL)<<16);
		MEMCPY(&_WinRTpp01[ 7].command, "Read: Set DMA transfer length 0x%X\n", 36);
		_WinRTpp01[10].port = Address;
		_WinRTpp01[10].value |= ((WINRT_TRACE_INFORMATIONAL)<<16);
		MEMCPY(&_WinRTpp01[11].command, "Read: Set DMA transfer address 0x%X\n", 37);
		_WinRTpp01[15].port = AMCC5933BusMasterControlStatus;//0x3c
		_WinRTpp01[20].value |= ((WINRT_TRACE_INFORMATIONAL)<<16);
		MEMCPY(&_WinRTpp01[21].command, "Read: Start DMA transfer, Wrote MCSR 0x%X\n", 51);
		_WinRTpp01[27].port = AMCC5933BusMasterControlStatus;//0x3c
		if (!WinRTProcessIoBufferDirect(hWinRTBuff, _WinRTpp01,sizeof(_WinRTpp01), &iWinRTlength))
			goto dmaReadFifoError;

	}
	return (ERROR_SUCCESS);

	dmaReadFifoError:                       // For errors
	Error = GetLastError();
	theApp.dlg->LabelError.SetWindowText("BufferReadFifo() failed "+Error);
	return (Error);
}
/*------------------------------------------------------------------------*/
void CBufferThread::WriteToCard(int WriteSize, bool StreamA)
{
	DWORD WaitTimeout;
	if(StreamA)
		CopyMemory(DmaBufferInfo.pVirtualAddress,DataBuffer1,WriteSize);
	else
		CopyMemory(DmaBufferInfo.pVirtualAddress,DataBuffer2,WriteSize);

	DmaWriteFifo(WriteSize);
		//wait for interrupt thread to signal transfer is complete
	WaitTimeout=WaitForSingleObject(hDMAReadCompleteEvent,5000);
	if(WaitTimeout == WAIT_TIMEOUT)
	{
		theApp.dlg->LabelError.SetWindowText("Buffer:WaitForDMAReadComplete failed, timeout"
					+ WaitTimeout);
	}
	ResetEvent(hDMAReadCompleteEvent);
}
/*------------------------------------------------------------------------*/
/////////////////////////////////////////////////////////////////////
//  DmaWriteFifo - sets up write DMA process,
//  Inputs: Length - Dma write  length (bytes)
//  Outputs:
//  Notes:
//////////////////////////////////////////////////////////////////////
DWORD CBufferThread::DmaWriteFifo(DWORD Length)
{
	DWORD       Error=0;                  // error return value
	DWORD       Address=0;                // address of WinRT common buffer (logical address)
	DWORD       iWinRTlength=0;           // length from the driver
	CString		buffer;


	if((Length < 4) || (Length > DmaBufferInfo.Length)) // check for valid length
	{
		buffer.Format(_T("DmaWriteFifo: Invalid Length %d valid range 4-%d"),
						  Length,
						  DmaBufferInfo.Length
						  );
		theApp.dlg->LabelError.SetWindowText(buffer);

		return (ERROR_INVALID_DATA);
	}


	if (Length & 0x3) // check that length can be converted into ULONG count
	{
		theApp.dlg->LabelError.SetWindowText("DmaWriteFifo: Invalid Length,  must be on a DWORD boundary");
		return (ERROR_INVALID_DATA);
	}
	// setup data in common buffer for write DMA

	Address = (DWORD)(DmaBufferInfo.LogicalAddress); //} get common buffer
													//} address for DMA (logical)

	//WinRT structure- set up MCSR and buffer
	static WINRT_CONTROL_ITEM _WinRTpp02[] =
		{//	command	param1	param2
			{DIM,	0x00040001,	 0x0},	// L:0 DmaMask
			{DIM,	0x00040001,	 0x0},	// L:1 StartDMA
			{DIM,	0x00040001,	 0x0},	// L:2 DmaControl
			{DIM,	0x00040001,	 0x0},	// L:3 Temp
			{OUTP_L,	0,	0},
			{OUTP_L,	0,	0},
			{WINRT_TRACE,	0x00000000,	0x00000025},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{WINRT_TRACE,	0x00000000,	0x00000026},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{INP_L,	0,	0},
			{MATH,	0x000D0002,	0x00024000},
			{MATH,	0x00080003,	0x00020000},
			{MATH,	0x00090002,	0x00030001},
			{MATH,	0x000D0002,	0x00021000},
			{WINRT_TRACE,	0x00000000,	0x00000035},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{MATH,	0x000D0002,	0x00022000},
			{OUTP_L,	0,	0},
		};
		_WinRTpp02[ 0].value = (ULONG)S5933_MCSR_DMA_MASK;//0x0000FF00
		_WinRTpp02[ 1].value = (ULONG)S5933_MCSR_READ_TRANSFER_ENABLE;//(1<<14)
		_WinRTpp02[ 4].port = AMCC5933MasterReadAddress;
		_WinRTpp02[ 4].value = Address;
		_WinRTpp02[ 5].port = AMCC5933MasterReadTransferCount;
		_WinRTpp02[ 5].value = Length;
		_WinRTpp02[ 6].port = Length;
		_WinRTpp02[ 6].value |= ((WINRT_TRACE_INFORMATIONAL )<<16);
		MEMCPY(&_WinRTpp02[ 7].command, "Write: Set DMA transfer length 0x%X\n", 37);
		_WinRTpp02[11].port = Address;
		_WinRTpp02[11].value |= ((WINRT_TRACE_INFORMATIONAL)<<16);
		MEMCPY(&_WinRTpp02[12].command, "Write: Set DMA transfer address 0x%X\n", 38);
		_WinRTpp02[16].port = AMCC5933BusMasterControlStatus;
		_WinRTpp02[21].value |= ((WINRT_TRACE_INFORMATIONAL)<<16);
		MEMCPY(&_WinRTpp02[22].command, "Write: Start DMA Transfer, Wrote MCSR 0x%X\n", 53);
		_WinRTpp02[28].port = AMCC5933BusMasterControlStatus;
		if (!WinRTProcessIoBufferDirect(hWinRTBuff, _WinRTpp02,sizeof(_WinRTpp02), &iWinRTlength))
			goto dmaWriteFifoError;

	return (ERROR_SUCCESS);

dmaWriteFifoError:
	Error = GetLastError();
	theApp.dlg->LabelError.SetWindowText("DmaWriteFifo() failed "
									+ Error);
	return (Error);
}
/*------------------------------------------------------------------------*/
void CBufferThread::Terminate()
{
	Terminated = true;
}