//---------------------------------------------------------------------------
/*
This unit contains the Interrupt control thread. The thread is created and
destroyed by the main thread.
The execute method waits for any of 5 interrupt events generated by the WinRT
deferred processing function, then responds to each.
a)Incoming mailbox interrupt - causes mailbox1 to be read, determines the cause
 and sets level threshold events.
b)DMA write interrupt - indicates a write transfer has been completed, sets
DMAWriteComplete event.
c)DMA read interrupt - indicates a read transfer has been completed, sets
DMAReadComplete event.
d) Outgoing mailbox interrupt  - no operation
e) AMCC error interrupt - no operation

*/
/*--Includes---------------------------------------------------------------------*/
//#include <vcl.h>
//#pragma hdrstop
#include <stdafx.h>
#include <winioctl.h>
#include "InterruptThread.h"
#include "R2D3Interrupt.h"
#include "R2D3DABCtl.h"
#include "amcc5933.h"
#include "winrtctl.h"
//#pragma package(smart_init)

/*--Defines----------------------------------------------------------------------*/
/*--Globals----------------------------------------------------------------------*/
extern AMCC_ADDRESS    AmccAddress;
extern HANDLE hDMAWriteCompleteEvent;
extern HANDLE hDMAReadCompleteEvent;
extern HANDLE hHiThreshIntEvent;
extern HANDLE hLoThreshIntEvent;
extern CRITICAL_SECTION  csStatusUpdate;
extern PAMCC_REGISTERS  pAmccRegisters;
extern bool Write,Read,ReadDisp;
extern HANDLE hWinRTInt;
extern __int64 curpos1;

extern WINRT_DMA_BUFFER_INFORMATION  DmaBufferInfo; //DMA buffer information
//extern CInterruptThread *pInterruptThread;

/*--Functions--------------------------------------------------------------------*/
// Stub
UINT CInterruptThread::mtstub1(LPVOID xthis)
{
	CInterruptThread *_this=(CInterruptThread *)xthis; 
	_this->Execute();
	return 0;  // or return the return value of Thread1() if you want
}
/*-------------------------------------------------------------------------------*/
CInterruptThread::CInterruptThread()
{
	Terminated = false;
	DataSourceA = false;
	DataSourceB = false;

	AfxBeginThread(mtstub1, this, THREAD_PRIORITY_HIGHEST, 0, 0);
}


/*-------------------------------------------------------------------------------*/
void CInterruptThread::Execute()
{
DWORD       WaitResult = 0;         // result of WaitForSingleObject
DWORD       Event=0;
DWORD Data;
extern HANDLE AmccWin32InterruptEvents[];
intcount=0;

while(!Terminated)
  {
		// wait for interrupt events
	WaitResult = WaitForMultipleObjects(5,AmccWin32InterruptEvents,FALSE,INFINITE);
	if(Terminated)
    break;

	Event = (WaitResult - WAIT_OBJECT_0);  // check which event occurred
	if (Event == AMCCMAILBOXREADINTERRUPT) //check for threshold changes
		{
		ReadMailBox(IMB4,Data);
		InterruptSource(Data);
		Beep(1500, 100);
		}
	if (Event == AMCCMAILBOXWRITEINTERRUPT)//process if confirmation of command required
		theApp.dlg->LabelError.SetWindowText("Interrupt:MailboxWrite interrupt received");
	if (Event == AMCCERRORINTERRUPT) //  do some error processing
		theApp.dlg->LabelError.SetWindowText("Interrupt:AMCC Error interrupt received");
	if (Event == AMCCDMAWRITEINTERRUPT) //
		{
			//acknowledge receipt of interrupt
		SetEvent(hDMAWriteCompleteEvent);
		}
	if (Event == AMCCDMAREADINTERRUPT) //
		{ //acknowledge receipt of interrupt
		SetEvent(hDMAReadCompleteEvent);
		}
	ResetEvent(AmccWin32InterruptEvents[Event]);
	}
}
/*-------------------------------------------------------------------------------*/

/*
ReadMailBox  - reads incoming mailbox, tests for thresholds and sets
events for hi and lo thresholds.
*/
DWORD CInterruptThread::ReadMailBox(DWORD Address,DWORD &Data)
{
	//DWORD       Data=0;                   // mail box data
	DWORD       Error=0;                  // error return value
	DWORD       iWinRTlength=0;           // length from the driver
	static WINRT_CONTROL_ITEM _WinRTpp33[] =
	{//	command	param1	param2
			{DIM,	0x00040001,	 0x0},	// L:0 Data
			{INP_LA,	0,	0},
			{MATH,	0x000D0000,	0x00004000},
			{WINRT_TRACE,	0x00000000,	0x0000001c},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
			{MATH,	0x000D0000,	0x00001000},
			{WINRT_TRACE,	0x00000000,	0x00000010},     		{NOP,	 0x0,	 0x0},
			{NOP,	 0x0,	 0x0},
		};
	_WinRTpp33[ 1].port = IN_MAILBOX_ADDRESS;
	_WinRTpp33[ 3].port = IN_MAILBOX_ADDRESS;
	_WinRTpp33[ 3].value |= ((WINRT_TRACE_INFORMATIONAL)<<16);
	MEMCPY(&_WinRTpp33[ 4].command, "Read MailBox register 0x%X\n", 28);
	_WinRTpp33[ 8].value |= ((WINRT_TRACE_INFORMATIONAL)<<16);
	MEMCPY(&_WinRTpp33[ 9].command, "Read data 0X%X\n", 16);

	if (!WinRTProcessIoBufferDirect(hWinRTInt, _WinRTpp33,sizeof(_WinRTpp33), &iWinRTlength))
		goto readMailBoxError;

	Data=(ULONG)_WinRTpp33[ 0].value;
	return (ERROR_SUCCESS);

	readMailBoxError:
	Error = GetLastError();
	theApp.dlg->LabelError.SetWindowText("ReadIntMailBox() failed " + Error);
	return (Error);
}

/*-------------------------------------------------------------------------------*/
void CInterruptThread::InterruptSource(DWORD MailboxData)
{
	if((MailboxData & STRA_THRESH)&&(MailboxData & UPPER_THRESH))//hi thresh A
		DataSourceA=true;
	else
		DataSourceA=false;
	if((MailboxData & STRB_THRESH)&&(MailboxData & UPPER_THRESH))//hi thresh B
		DataSourceB=true;
	else
		DataSourceB=false;

	if(DataSourceA||DataSourceB)
	{
		SetEvent(hHiThreshIntEvent);
		theApp.dlg->gInterruptCommand&= ~S5933_INTCSR_INCOMING_INTERRUPT;
		EnableAmccInterrupts(theApp.dlg->gInterruptCommand);
	}
}
/*-------------------------------------------------------------------------------*/
void CInterruptThread::Terminate()
{
	Terminated = true;
}
